@@ -21,12 +21,16 @@
 #include "extractorengine.h"
 #include "extractor.h"
 #include "extractorrepository.h"
-#include "genericpdfextractor.h"
+#include "genericpdfextractor_p.h"
+#include "genericpkpassextractor_p.h"
+#include "genericuic918extractor_p.h"
 #include "htmldocument.h"
+#include "iatabcbpparser.h"
 #include "jsonlddocument.h"
 #include "logging.h"
 #include "pdfdocument.h"
-#include "structureddataextractor.h"
+#include "structureddataextractor_p.h"
+#include "uic9183parser.h"
 
 #include "jsapi/barcode.h"
 #include "jsapi/context.h"
@@ -38,11 +42,10 @@
 #include <KCalCore/ICalFormat>
 #endif
 
-#include <KPkPass/Barcode>
-#include <KPkPass/BoardingPass>
-#include <KPkPass/Location>
+#include <KPkPass/Pass>
 
 #include <KMime/Content>
+#include <KMime/Message>
 
 #include <QDateTime>
 #include <QFile>
@@ -53,6 +56,8 @@
 #include <QJSEngine>
 #include <QJSValueIterator>
 
+#include <cstring>
+
 using namespace KItinerary;
 
 namespace KItinerary {
@@ -74,15 +79,13 @@ public:
 
     void executeScript(const Extractor *extractor);
     void processScriptResult(const QJSValue &result);
-    void extractPass();
-    void extractBoardingPass(QJsonObject &resFor);
-    void extractEventTicketPass(QJsonObject &resFor);
 
     std::vector<const Extractor*> m_extractors;
     JsApi::Barcode *m_barcodeApi = nullptr;
     JsApi::Context *m_context = nullptr;
     JsApi::JsonLd *m_jsonLdApi = nullptr;
     QString m_text;
+    QByteArray m_data;
     std::unique_ptr<HtmlDocument, std::function<void(HtmlDocument*)>> m_htmlDoc;
     std::unique_ptr<PdfDocument, std::function<void(PdfDocument*)>> m_pdfDoc;
     std::unique_ptr<KPkPass::Pass, std::function<void(KPkPass::Pass*)>> m_pass;
@@ -91,6 +94,7 @@ public:
 #endif
     KMime::Content *m_mimeContent = nullptr;
     KMime::Content *m_mimeContext = nullptr;
+    std::unique_ptr<KMime::Content> m_ownedMimeContent;
     GenericPdfExtractor m_genericPdfExtractor;
     QJsonArray m_result;
     QJSEngine m_engine;
@@ -138,11 +142,13 @@ void ExtractorEngine::clear()
     d->m_result = {};
     d->m_mimeContext = nullptr;
     d->m_context->m_senderDate = {};
+    d->m_ownedMimeContent.reset();
 }
 
 void ExtractorEnginePrivate::resetContent()
 {
     m_text.clear();
+    m_data.clear();
     m_pdfDoc.reset();
     m_htmlDoc.reset();
     m_pass.reset();
@@ -152,11 +158,6 @@ void ExtractorEnginePrivate::resetContent()
     m_mimeContent = nullptr;
 }
 
-void ExtractorEngine::setExtractors(std::vector<const Extractor*> &&extractors)
-{
-    d->m_extractors = extractors;
-}
-
 void ExtractorEngine::setText(const QString &text)
 {
     d->m_text = text;
@@ -209,7 +210,9 @@ void ExtractorEnginePrivate::setContent(KMime::Content *content)
 #ifdef HAVE_KCAL
         m_calendar.reset(new KCalCore::MemoryCalendar(QTimeZone()));
         KCalCore::ICalFormat format;
-        if (!format.fromRawString(m_calendar, content->decodedContent())) {
+        if (format.fromRawString(m_calendar, content->decodedContent())) {
+            m_calendar->setProductId(format.loadedProductId());
+        } else {
             m_calendar.reset();
         }
 #endif
@@ -224,24 +227,142 @@ void ExtractorEnginePrivate::setContent(KMime::Content *content)
     m_mimeContent = (ct && ct->isMultipart()) ? content : nullptr;
 }
 
+static bool contentStartsWith(const QByteArray &data, char s)
+{
+    for (const auto c : data) {
+        if (std::isspace(c)) {
+            continue;
+        }
+        return c == s;
+    }
+    return false;
+}
+
+static bool contentStartsWith(const QByteArray &data, const char *str)
+{
+    auto it = data.begin();
+    while (it != data.end() && std::isspace(*it)) {
+        ++it;
+    }
+
+    const auto len = std::strlen(str);
+    if ((int)len >= std::distance(it, data.end())) {
+        return false;
+    }
+    return std::strncmp(it, str, len) == 0;
+}
+
+static bool contentMightBeEmail(const QByteArray &data)
+{
+    for (const auto c : data) {
+        if (std::isalpha(c) || c == '-') {
+            continue;
+        }
+        return c == ':';
+    }
+    return false;
+}
+
+void ExtractorEngine::setData(const QByteArray &data, const QString &fileName)
+{
+    // let's not even try to parse anything with implausible size
+    if (data.size() <= 4 || data.size() > 4000000) {
+        return;
+    }
+
+    if (fileName.endsWith(QLatin1String(".pkpass"), Qt::CaseInsensitive) || strncmp(data.constData(), "PK\x03\x04", 4) == 0) {
+        d->m_pass = make_owning_ptr(KPkPass::Pass::fromData(data));
+        return;
+    }
+
+    if (fileName.endsWith(QLatin1String(".pdf"), Qt::CaseInsensitive) ||  strncmp(data.constData(), "%PDF", 4) == 0) {
+        d->m_pdfDoc = make_owning_ptr(PdfDocument::fromData(data));
+        return;
+    }
+
+    if (fileName.endsWith(QLatin1String(".html"), Qt::CaseInsensitive)
+        || fileName.endsWith(QLatin1String(".htm"), Qt::CaseInsensitive)
+        || contentStartsWith(data, '<'))
+    {
+        d->m_htmlDoc = make_owning_ptr(HtmlDocument::fromData(data));
+        return;
+    }
+
+    if (fileName.endsWith(QLatin1String(".ics"), Qt::CaseInsensitive)
+        || fileName.endsWith(QLatin1String(".ical"), Qt::CaseInsensitive)
+        || contentStartsWith(data, "BEGIN:VCALENDAR"))
+    {
+#ifdef HAVE_KCAL
+        d->m_calendar.reset(new KCalCore::MemoryCalendar(QTimeZone()));
+        KCalCore::ICalFormat format;
+        if (format.fromRawString(d->m_calendar, data)) {
+            d->m_calendar->setProductId(format.loadedProductId());
+            return;
+        }
+        qCDebug(Log) << "Failed to parse iCal content.";
+        d->m_calendar.reset();
+#else
+        qCDebug(Log) << "Trying to exctract ical file, but ical support is not enabled.";
+#endif
+    }
+
+    if (fileName.endsWith(QLatin1String(".txt"), Qt::CaseInsensitive)) {
+        d->m_text = QString::fromUtf8(data);
+        return;
+    }
+
+    if (fileName.endsWith(QLatin1String(".eml"), Qt::CaseInsensitive)
+        || fileName.endsWith(QLatin1String(".mbox"), Qt::CaseInsensitive)
+        || contentMightBeEmail(data))
+    {
+        d->m_ownedMimeContent.reset(new KMime::Message);
+        d->m_ownedMimeContent->setContent(data);
+        d->m_ownedMimeContent->parse();
+        setContent(d->m_ownedMimeContent.get());
+        return;
+    }
+
+    if (fileName.endsWith(QLatin1String(".json"), Qt::CaseInsensitive)
+        || fileName.endsWith(QLatin1String(".jsonld"), Qt::CaseInsensitive)
+        || contentStartsWith(data, '{') || contentStartsWith(data, '['))
+    {
+        // pass through JSON data, so the using code can apply post-processing to that
+        const auto doc = QJsonDocument::fromJson(data);
+        if (doc.isObject()) {
+            d->m_result.push_back(doc.object());
+            return;
+        } else if (doc.isArray()) {
+            d->m_result = doc.array();
+            return;
+        }
+    }
+
+    d->m_data = data;
+}
+
 void ExtractorEnginePrivate::setContext(KMime::Content *context)
 {
     m_mimeContext = context;
-    auto dateHdr = context->header<KMime::Headers::Date>();
-    while (!dateHdr && context->parent()) {
-        context = context->parent();
-        dateHdr = context->header<KMime::Headers::Date>();
-    }
-    if (dateHdr) {
-        setContextDate(dateHdr->dateTime());
+    if (context) {
+        auto dateHdr = context->header<KMime::Headers::Date>();
+        while (!dateHdr && context->parent()) {
+            context = context->parent();
+            dateHdr = context->header<KMime::Headers::Date>();
+        }
+        if (dateHdr) {
+            setContextDate(dateHdr->dateTime());
+            return;
+        }
     }
+
+    setContextDate({});
 }
 
 void ExtractorEnginePrivate::setContextDate(const QDateTime &dt)
 {
     m_context->m_senderDate = dt;
     m_jsonLdApi->setContextDate(dt);
-    m_barcodeApi->setContextDate(dt.date());
+    m_barcodeApi->setContextDate(dt);
     m_genericPdfExtractor.setContextDate(dt);
 }
 
@@ -273,7 +394,9 @@ QJsonArray ExtractorEngine::extract()
 
 void ExtractorEnginePrivate::extractRecursive(KMime::Content *content)
 {
-    for (const auto child : content->contents()) {
+    QJsonArray aggregatedResult;
+    const auto children = content->contents();
+    for (const auto child : children) {
         resetContent();
         setContent(child);
         if (m_mimeContent) {
@@ -281,7 +404,13 @@ void ExtractorEnginePrivate::extractRecursive(KMime::Content *content)
         } else {
             extractDocument();
         }
+
+        // the extractor takes early exits if data has been found, so make it look like that isn't the case
+        std::copy(m_result.begin(), m_result.end(), std::back_inserter(aggregatedResult));
+        m_result = {};
     }
+
+    m_result = std::move(aggregatedResult);
 }
 
 void ExtractorEnginePrivate::extractDocument()
@@ -293,28 +422,29 @@ void ExtractorEnginePrivate::extractDocument()
     }
 
     // custom extractors
+    m_extractors.clear();
     if (m_pass) {
         m_extractors = m_repo.extractorsForPass(m_pass.get());
-    } else if (m_mimeContext) {
+#ifdef HAVE_KCAL
+    } else if (m_calendar) {
+        m_extractors = m_repo.extractorsForCalendar(m_calendar);
+#endif
+    }
+    if (m_extractors.empty() && m_mimeContext) {
         m_extractors = m_repo.extractorsForMessage(m_mimeContext);
     }
     extractCustom();
 
     // generic extractors
     extractGeneric();
-
-    // check if generic extractors identified documents we have custom extractors for
-    m_extractors = m_repo.extractorsForJsonLd(m_result);
-    extractCustom();
 }
 
 void ExtractorEnginePrivate::extractStructured()
 {
     if (m_htmlDoc) {
         qCDebug(Log) << "Looking for structured annotations...";
-        for (const auto &v : StructuredDataExtractor::extract(m_htmlDoc.get())) {
-            m_result.push_back(v);
-        }
+        const auto res = StructuredDataExtractor::extract(m_htmlDoc.get());
+        std::copy(res.begin(), res.end(), std::back_inserter(m_result));
     }
 }
 
@@ -331,6 +461,9 @@ void ExtractorEnginePrivate::extractCustom()
                 if (m_text.isEmpty() && m_htmlDoc) {
                     m_text = m_htmlDoc->root().recursiveContent();
                 }
+                if (m_text.isEmpty() && !m_data.isEmpty()) {
+                    m_text = QString::fromUtf8(m_data);
+                }
 
                 if (!m_text.isEmpty()) {
                     executeScript(extractor);
@@ -369,9 +502,48 @@ void ExtractorEnginePrivate::extractCustom()
 void ExtractorEnginePrivate::extractGeneric()
 {
     if (m_pass) {
-        extractPass();
+        if (m_result.size() > 1) { // a pkpass file contains exactly one boarding pass
+            return;
+        }
+        if (m_result.isEmpty()) {
+            m_result.push_back(QJsonObject());
+        }
+
+        auto res = m_result.at(0).toObject();
+        GenericPkPassExtractor::extract(m_pass.get(), res);
+        m_result[0] = res;
     } else if (m_pdfDoc && m_result.isEmpty()) {
-        m_genericPdfExtractor.extract(m_pdfDoc.get(), m_result);
+        QJsonArray genericResult;
+        m_genericPdfExtractor.extract(m_pdfDoc.get(), genericResult);
+
+        // check if generic extractors identified documents we have custom extractors for
+        m_extractors = m_repo.extractorsForJsonLd(genericResult);
+        extractCustom();
+
+        // check the unrecognized (vendor-specific) barcodes, if any
+        const auto unrecognizedCodes = m_genericPdfExtractor.unrecognizedBarcodes();
+        for (const auto &code : unrecognizedCodes) {
+            m_extractors = m_repo.extractorsForBarcode(code);
+            extractCustom();
+        }
+
+        // if none of that found something, take the generic extractor result as-is
+        if (m_result.isEmpty()) {
+            m_result = genericResult;
+        }
+    } else if (!m_text.isEmpty() && m_result.isEmpty()) {
+        if (IataBcbpParser::maybeIataBcbp(m_text)) {
+            const auto res = IataBcbpParser::parse(m_text, m_context->m_senderDate.date());
+            m_result = JsonLdDocument::toJson(res);
+        }
+    } else if (!m_data.isEmpty() && m_result.isEmpty()) {
+        if (Uic9183Parser::maybeUic9183(m_data)) {
+            GenericUic918Extractor::extract(m_data, m_result, m_context->m_senderDate);
+            return;
+        }
+        // try again as text
+        m_text = QString::fromUtf8(m_data);
+        extractGeneric();
     }
 }
 
@@ -390,8 +562,8 @@ void ExtractorEnginePrivate::executeScript(const Extractor *extractor)
 
     auto result = m_engine.evaluate(QString::fromUtf8(f.readAll()), f.fileName());
     if (result.isError()) {
-        qCWarning(Log) << "Script parsing error in" << result.property(QLatin1String("fileName")).toString()
-                                << ':' << result.property(QLatin1String("lineNumber")).toInt() << result.toString();
+        qCWarning(Log) << "Script parsing error in" << result.property(QStringLiteral("fileName")).toString()
+                                << ':' << result.property(QStringLiteral("lineNumber")).toInt() << result.toString();
         return;
     }
 
@@ -418,7 +590,8 @@ void ExtractorEnginePrivate::executeScript(const Extractor *extractor)
             break;
         case Extractor::ICal:
 #ifdef HAVE_KCAL
-            for (const auto &event : m_calendar->events()) {
+            const auto events = m_calendar->events();
+            for (const auto &event : events) {
                 processScriptResult(mainFunc.call({m_engine.toScriptValue(*event.data())}));
             }
 #endif
@@ -433,8 +606,8 @@ void ExtractorEnginePrivate::executeScript(const Extractor *extractor)
 void ExtractorEnginePrivate::processScriptResult(const QJSValue &result)
 {
     if (result.isError()) {
-        qCWarning(Log) << "Script execution error in" << result.property(QLatin1String("fileName")).toString()
-                                << ':' << result.property(QLatin1String("lineNumber")).toInt() << result.toString();
+        qCWarning(Log) << "Script execution error in" << result.property(QStringLiteral("fileName")).toString()
+                                << ':' << result.property(QStringLiteral("lineNumber")).toInt() << result.toString();
         return;
     }
 
@@ -452,147 +625,3 @@ void ExtractorEnginePrivate::processScriptResult(const QJSValue &result)
         qCWarning(Log) << "Invalid result type from script";
     }
 }
-
-void ExtractorEnginePrivate::extractPass()
-{
-    if (m_result.size() > 1) { // a pkpass file contains exactly one boarding pass
-        return;
-    }
-
-    if (m_result.isEmpty()) { // no script run, so we need to create the top-level element ourselves
-        QJsonObject res;
-        QJsonObject resFor;
-        if (auto boardingPass = qobject_cast<KPkPass::BoardingPass*>(m_pass.get())) {
-            switch (boardingPass->transitType()) {
-                case KPkPass::BoardingPass::Air:
-                    res.insert(QLatin1String("@type"), QLatin1String("FlightReservation"));
-                    resFor.insert(QLatin1String("@type"), QLatin1String("Flight"));
-                    break;
-                // TODO expand once we have test files for train tickets
-                default:
-                    return;
-            }
-        } else {
-            switch (m_pass->type()) {
-                case KPkPass::Pass::EventTicket:
-                    res.insert(QLatin1String("@type"), QLatin1String("EventReservation"));
-                    resFor.insert(QLatin1String("@type"), QLatin1String("Event"));
-                    break;
-                default:
-                    return;
-            }
-        }
-        res.insert(QLatin1String("reservationFor"), resFor);
-        m_result.push_back(res);
-    }
-
-    // extract structured data from a pkpass, if the extractor script hasn't done so already
-    auto res = m_result.at(0).toObject();
-    auto resFor = res.value(QLatin1String("reservationFor")).toObject();
-    switch (m_pass->type()) {
-        case KPkPass::Pass::BoardingPass:
-            extractBoardingPass(resFor);
-            break;
-        case KPkPass::Pass::EventTicket:
-            extractEventTicketPass(resFor);
-            break;
-        default:
-            return;
-    }
-
-    // barcode contains the ticket token
-    if (!m_pass->barcodes().isEmpty() && !res.contains(QLatin1String("reservedTicket"))) {
-        const auto barcode = m_pass->barcodes().at(0);
-        QString token;
-        switch (barcode.format()) {
-            case KPkPass::Barcode::QR:
-                token += QLatin1String("qrCode:");
-                break;
-            case KPkPass::Barcode::Aztec:
-                token += QLatin1String("aztecCode:");
-                break;
-            default:
-                break;
-        }
-        token += barcode.message();
-        QJsonObject ticket;
-        ticket.insert(QLatin1String("@type"), QLatin1String("Ticket"));
-        ticket.insert(QLatin1String("ticketToken"), token);
-        res.insert(QLatin1String("reservedTicket"), ticket);
-    }
-
-    res.insert(QLatin1String("reservationFor"), resFor);
-
-    // associate the pass with the result, so we can find the pass again for display
-    if (!m_pass->passTypeIdentifier().isEmpty() && !m_pass->serialNumber().isEmpty()) {
-        res.insert(QLatin1String("pkpassPassTypeIdentifier"), m_pass->passTypeIdentifier());
-        res.insert(QLatin1String("pkpassSerialNumber"), m_pass->serialNumber());
-    }
-
-    m_result[0] = res;
-}
-
-void ExtractorEnginePrivate::extractBoardingPass(QJsonObject &resFor)
-{
-    // "relevantDate" is the best guess for the boarding time
-    if (m_pass->relevantDate().isValid() && !resFor.contains(QLatin1String("boardingTime"))) {
-        resFor.insert(QLatin1String("boardingTime"), m_pass->relevantDate().toString(Qt::ISODate));
-    }
-    // look for common field names containing the boarding time, if we still have no idea
-    if (!resFor.contains(QLatin1String("boardingTime"))) {
-        for (const auto &field : m_pass->fields()) {
-            if (!field.key().contains(QLatin1String("boarding"), Qt::CaseInsensitive)) {
-                continue;
-            }
-            const auto time = QTime::fromString(field.value().toString());
-            if (time.isValid()) {
-                // this misses date, but the postprocessor will fill that in
-                resFor.insert(QLatin1String("boardingTime"), QDateTime(QDate(1, 1, 1), time).toString(Qt::ISODate));
-                break;
-            }
-        }
-    }
-
-    // location is the best guess for the departure airport geo coordinates
-    auto depAirport = resFor.value(QLatin1String("departureAirport")).toObject();
-    if (depAirport.isEmpty()) {
-        depAirport.insert(QLatin1String("@type"), QLatin1String("Airport"));
-    }
-    auto depGeo = depAirport.value(QLatin1String("geo")).toObject();
-    if (m_pass->locations().size() == 1 && depGeo.isEmpty()) {
-        const auto loc = m_pass->locations().at(0);
-        depGeo.insert(QLatin1String("@type"), QLatin1String("GeoCoordinates"));
-        depGeo.insert(QLatin1String("latitude"), loc.latitude());
-        depGeo.insert(QLatin1String("longitude"), loc.longitude());
-        depAirport.insert(QLatin1String("geo"), depGeo);
-        resFor.insert(QLatin1String("departureAirport"), depAirport);
-    }
-}
-
-void ExtractorEnginePrivate::extractEventTicketPass(QJsonObject &resFor)
-{
-    if (!resFor.contains(QLatin1String("name"))) {
-        resFor.insert(QLatin1String("name"), m_pass->description());
-    }
-
-    // "relevantDate" is the best guess for the start time
-    if (m_pass->relevantDate().isValid() && !resFor.contains(QLatin1String("startDate"))) {
-        resFor.insert(QLatin1String("startDate"), m_pass->relevantDate().toString(Qt::ISODate));
-    }
-
-    // location is the best guess for the venue
-    auto venue = resFor.value(QLatin1String("location")).toObject();
-    if (venue.isEmpty()) {
-        venue.insert(QLatin1String("@type"), QLatin1String("Place"));
-    }
-    auto geo = venue.value(QLatin1String("geo")).toObject();
-    if (!m_pass->locations().isEmpty() && geo.isEmpty()) {
-        const auto loc = m_pass->locations().at(0);
-        geo.insert(QLatin1String("@type"), QLatin1String("GeoCoordinates"));
-        geo.insert(QLatin1String("latitude"), loc.latitude());
-        geo.insert(QLatin1String("longitude"), loc.longitude());
-        venue.insert(QLatin1String("geo"), geo);
-        venue.insert(QLatin1String("name"), loc.relevantText());
-        resFor.insert(QLatin1String("location"), venue);
-    }
-}
