@@ -22,9 +22,15 @@
 
 #include "kitinerary_export.h"
 
+#include <QSharedPointer>
+
 #include <memory>
 #include <vector>
 
+namespace KCalCore {
+class Calendar;
+}
+
 namespace KMime {
 class Content;
 }
@@ -34,6 +40,7 @@ class Pass;
 }
 
 class QJsonArray;
+class QString;
 
 namespace KItinerary {
 
@@ -49,7 +56,7 @@ class ExtractorRepository
 public:
     ExtractorRepository();
     ~ExtractorRepository();
-    ExtractorRepository(ExtractorRepository &&);
+    ExtractorRepository(ExtractorRepository &&) noexcept;
     ExtractorRepository(const ExtractorRepository &) = delete;
 
     /** Finds matching extractors for the given message part. */
@@ -58,6 +65,10 @@ public:
     std::vector<const Extractor *> extractorsForPass(KPkPass::Pass *pass) const;
     /** Finds matching extractors for the given JSON-LD data provided by generic extractors. */
     std::vector<const Extractor *> extractorsForJsonLd(const QJsonArray &data) const;
+    /** Finds matching extractors for the given barcode string. */
+    std::vector<const Extractor *> extractorsForBarcode(const QString &code) const;
+    /** Find matching extractors for the given iCal calendar. */
+    std::vector<const Extractor *> extractorsForCalendar(const QSharedPointer<KCalCore::Calendar> &cal) const;
 
 private:
     ExtractorRepositoryPrivate* d;
