@@ -12,11 +12,12 @@
     License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "uic9183parser.h"
 #include "logging.h"
+#include "rct2ticket.h"
 
 #include <QDateTime>
 #include <QDebug>
@@ -28,20 +29,6 @@
 
 using namespace KItinerary;
 
-static int asciiToInt(const char *s, int size)
-{
-    if (!s) {
-        return 0;
-    }
-
-    int v = 0;
-    for (int i = 0; i < size; ++i) {
-        v *= 10;
-        v += (*(s + i)) - '0';
-    }
-    return v;
-}
-
 namespace KItinerary {
 
 class Uic9183Block
@@ -109,41 +96,7 @@ public:
     Uic9183Block findBlock(const char name[6]) const;
 
     QByteArray m_payload;
-};
-
-// 2x field line, number as ascii text
-// 2x field column
-// 2x field height
-// 2x field width
-// 1x field format
-// 4x text length
-// Nx text content
-class Rct2TicketField
-{
-public:
-    Rct2TicketField() = default;
-    Rct2TicketField(const char *data);
-    // size of the field data, not size of the text content
-    int size() const;
-
-    int row() const;
-    int column() const;
-    int height() const;
-    int width() const;
-    QString text() const;
-
-private:
-    const char *m_data = nullptr;
-};
-
-class Rct2TicketPrivate : public QSharedData
-{
-public:
-    QString fieldText(int row, int column, int width, int height = 1) const;
-    QDate firstDayOfValidity() const;
-    QDateTime parseTime(const QString &dateStr, const QString &timeStr) const;
-
-    Uic9183Block block;
+    QDateTime m_contextDt;
 };
 }
 
@@ -234,175 +187,6 @@ Uic9183Block Uic9183ParserPrivate::findBlock(const char name[6]) const
 }
 
 
-Rct2TicketField::Rct2TicketField(const char *data)
-    : m_data(data)
-{
-}
-
-int Rct2TicketField::size() const
-{
-    return asciiToInt(m_data + 9, 4) + 13;
-}
-
-int Rct2TicketField::row() const
-{
-    return asciiToInt(m_data, 2);
-}
-
-int Rct2TicketField::column() const
-{
-    return asciiToInt(m_data + 2, 2);
-}
-
-int Rct2TicketField::height() const
-{
-    return asciiToInt(m_data + 4, 2);
-}
-
-int Rct2TicketField::width() const
-{
-    return asciiToInt(m_data + 6, 2);
-}
-
-QString Rct2TicketField::text() const
-{
-    return QString::fromUtf8(m_data + 13, asciiToInt(m_data + 9, 4));
-}
-
-QString Rct2TicketPrivate::fieldText(int row, int column, int width, int height) const
-{
-    QString s;
-    for (int i = 20; i < block.size();) {
-        Rct2TicketField f(block.data() + i);
-        i += f.size();
-
-        if (f.row() + f.height() - 1 < row || f.row() > row + height - 1) {
-            continue;
-        }
-        if (f.column() + f.width() - 1 < column || f.column() > column + width - 1) {
-            continue;
-        }
-        //qDebug() << "Field:" << f.height() << f.column() << f.height() << f.width() << f.size() << f.text();
-
-        // split field into lines
-        // TODO this needs to follow the RCT2 word-wrapping algorithm?
-        const auto content = f.text();
-        const auto lines = content.splitRef(QLatin1Char('\n'));
-
-        // cut out the right part of the line
-        for (int i = 0; i < lines.size(); ++i) {
-            if (f.row() + i < row) {
-                continue;
-            }
-            if (f.row() + i > row + height - 1) {
-                break;
-            }
-
-            // TODO also truncate by w
-            const auto offset = column - f.column();
-            if (offset >= 0) {
-                s += lines.at(i).mid(offset).left(width);
-            } else {
-                s += lines.at(i); // TODO left padding by offset, truncate by width + offset
-            }
-        }
-    }
-    //qDebug() << "Result:" << x << y << w << h << s;
-    return s;
-}
-
-QDate Rct2TicketPrivate::firstDayOfValidity() const
-{
-    const auto f = fieldText(3, 1, 48);
-    const auto it = std::find_if(f.begin(), f.end(), [](QChar c) { return c.isDigit(); });
-    if (it == f.end()) {
-        return {};
-    }
-
-    const auto dtStr = f.midRef(std::distance(f.begin(), it));
-    auto dt = QDate::fromString(dtStr.left(10).toString(), QStringLiteral("dd.MM.yyyy"));
-    if (dt.isValid()) {
-        return dt;
-    }
-    dt = QDate::fromString(dtStr.left(8).toString(), QStringLiteral("dd.MM.yy"));
-    if (dt.isValid()) {
-        if (dt.year() < 2000) {
-            dt.setDate(dt.year() + 100, dt.month(), dt.day());
-        }
-        return dt;
-    }
-    dt = QDate::fromString(dtStr.left(4).toString(), QStringLiteral("yyyy"));
-    return dt;
-}
-
-QDateTime Rct2TicketPrivate::parseTime(const QString &dateStr, const QString &timeStr) const
-{
-    const auto d = QDate::fromString(dateStr, QStringLiteral("dd.MM"));
-    const auto t = QTime::fromString(timeStr, QStringLiteral("hh:mm"));
-    return QDateTime({firstDayOfValidity().year(), d.month(), d.day()}, t);
-}
-
-
-// 6x "U_TLAY"
-// 2x version (always "01")
-// 4x record length, numbers as ASCII text
-// 4x ticket layout type ("RCT2")
-// 4x field count
-// Nx fields (see Rct2TicketField)
-Rct2Ticket::Rct2Ticket()
-    : d(new Rct2TicketPrivate)
-{
-}
-
-Rct2Ticket::Rct2Ticket(Uic9183Block block)
-    : d(new Rct2TicketPrivate)
-{
-    d->block = block;
-    qDebug() << QByteArray(block.data(), block.size());
-}
-
-Rct2Ticket::Rct2Ticket(const Rct2Ticket&) = default;
-Rct2Ticket::~Rct2Ticket() = default;
-Rct2Ticket& Rct2Ticket::operator=(const Rct2Ticket&) = default;
-
-bool Rct2Ticket::isValid() const
-{
-    return !d->block.isNull() && d->block.size() > 34
-        && std::strncmp(d->block.data() + 6, "01", 2) == 0
-        && std::strncmp(d->block.data() + 12, "RCT2", 4) == 0;
-}
-
-QDate Rct2Ticket::firstDayOfValidity() const
-{
-    return d->firstDayOfValidity();
-}
-
-QDateTime Rct2Ticket::outboundDepartureTime() const
-{
-    return d->parseTime(d->fieldText(6, 1, 5), d->fieldText(6, 7, 5));
-}
-
-QDateTime Rct2Ticket::outboundArrivalTime() const
-{
-    return d->parseTime(d->fieldText(6, 52, 5), d->fieldText(6, 58, 5));
-}
-
-QString Rct2Ticket::outboundDepartureStation() const
-{
-    return d->fieldText(6, 13, 17).trimmed();
-}
-
-QString Rct2Ticket::outboundArrivalStation() const
-{
-    return d->fieldText(6, 34, 17).trimmed();
-}
-
-QString Rct2Ticket::outboundClass() const
-{
-    return d->fieldText(6, 66, 5).trimmed();
-}
-
-
 Uic9183Parser::Uic9183Parser()
     : d(new Uic9183ParserPrivate)
 {
@@ -412,6 +196,11 @@ Uic9183Parser::Uic9183Parser(const Uic9183Parser&) = default;
 Uic9183Parser::~Uic9183Parser() = default;
 Uic9183Parser& Uic9183Parser::operator=(const Uic9183Parser&) = default;
 
+void Uic9183Parser::setContextDate(const QDateTime &contextDt)
+{
+    d->m_contextDt = contextDt;
+}
+
 void Uic9183Parser::parse(const QByteArray &data)
 {
     d->m_payload.clear();
@@ -527,7 +316,7 @@ Person Uic9183Parser::person() const
     // RCT2 tickets
     const auto rct2 = rct2Ticket();
     if (rct2.isValid()) {
-        const auto name = rct2.d->fieldText(0, 52, 19);
+        const auto name = rct2.passengerName();
         if (!name.isEmpty()) {
             Person p;
             p.setName(name);
@@ -570,7 +359,10 @@ QString Uic9183Parser::outboundArrivalStationId() const
 
 Rct2Ticket Uic9183Parser::rct2Ticket() const
 {
-    return Rct2Ticket(d->findBlock("U_TLAY"));
+    const auto block = d->findBlock("U_TLAY");
+    Rct2Ticket rct2(block.data(), block.size());
+    rct2.setContextDate(d->m_contextDt);
+    return rct2;
 }
 
 QVariant Uic9183Parser::rct2TicketVariant() const
